@@ -1,7 +1,7 @@
 /// \file
 // Range v3 library
 //
-//  Copyright Eric Niebler 2013-2014
+//  Copyright Eric Niebler 2013-present
 //  Copyright Casey Carter 2016
 //
 //  Use, modification and distribution is subject to the
@@ -156,6 +156,12 @@ namespace ranges
 #define RANGES_CXX_COROUTINES_14 0L
 #define RANGES_CXX_COROUTINES_17 0L
 #define RANGES_CXX_COROUTINES_TS1 201703L
+#define RANGES_CXX_DEDUCTION_GUIDES_11 0L
+#define RANGES_CXX_DEDUCTION_GUIDES_14 0L
+#define RANGES_CXX_DEDUCTION_GUIDES_17 201606L
+#define RANGES_CXX_IF_CONSTEXPR_11 0L
+#define RANGES_CXX_IF_CONSTEXPR_14 0L
+#define RANGES_CXX_IF_CONSTEXPR_17 201606L
 
 #if defined(_MSC_VER) && !defined(__clang__)
 #if _MSC_VER >= 1900
@@ -184,18 +190,23 @@ namespace ranges
 
 #define RANGES_DIAGNOSTIC_PUSH __pragma(warning(push))
 #define RANGES_DIAGNOSTIC_POP __pragma(warning(pop))
-#define RANGES_DIAGNOSTIC_IGNORE(X) __pragma(warning(disable:X))
+#define RANGES_DIAGNOSTIC_IGNORE_PRAGMAS __pragma(warning(disable:4068))
+#define RANGES_DIAGNOSTIC_IGNORE(X) RANGES_DIAGNOSTIC_IGNORE_PRAGMAS __pragma(warning(disable:X))
 #define RANGES_DIAGNOSTIC_IGNORE_SHADOWING RANGES_DIAGNOSTIC_IGNORE(4456)
-#define RANGES_DIAGNOSTIC_IGNORE_PRAGMAS RANGES_DIAGNOSTIC_IGNORE(4068)
 #define RANGES_DIAGNOSTIC_IGNORE_INDENTATION
 #define RANGES_DIAGNOSTIC_IGNORE_UNDEFINED_INTERNAL
 #define RANGES_DIAGNOSTIC_IGNORE_MISMATCHED_TAGS RANGES_DIAGNOSTIC_IGNORE(4099)
 #define RANGES_DIAGNOSTIC_IGNORE_GLOBAL_CONSTRUCTORS
 #define RANGES_DIAGNOSTIC_IGNORE_SIGN_CONVERSION
 #define RANGES_DIAGNOSTIC_IGNORE_UNNEEDED_INTERNAL
+#define RANGES_DIAGNOSTIC_IGNORE_UNNEEDED_MEMBER
 #define RANGES_DIAGNOSTIC_IGNORE_ZERO_LENGTH_ARRAY
 #define RANGES_DIAGNOSTIC_IGNORE_CXX17_COMPAT
 #define RANGES_DIAGNOSTIC_IGNORE_FLOAT_EQUAL
+#define RANGES_DIAGNOSTIC_IGNORE_MISSING_BRACES
+#define RANGES_DIAGNOSTIC_IGNORE_UNDEFINED_FUNC_TEMPLATE
+#define RANGES_DIAGNOSTIC_IGNORE_INCONSISTENT_OVERRIDE
+#define RANGES_DIAGNOSTIC_IGNORE_RANGE_LOOP_ANALYSIS
 
 #else // ^^^ defined(_MSC_VER) ^^^ / vvv !defined(_MSC_VER) vvv
 // Generic configuration using SD-6 feature test macros with fallback to __cplusplus
@@ -203,32 +214,28 @@ namespace ranges
 #define RANGES_PRAGMA(X) _Pragma(#X)
 #define RANGES_DIAGNOSTIC_PUSH RANGES_PRAGMA(GCC diagnostic push)
 #define RANGES_DIAGNOSTIC_POP RANGES_PRAGMA(GCC diagnostic pop)
-#define RANGES_DIAGNOSTIC_IGNORE(X) RANGES_PRAGMA(GCC diagnostic ignored X)
+#define RANGES_DIAGNOSTIC_IGNORE_PRAGMAS RANGES_PRAGMA(GCC diagnostic ignored "-Wpragmas")
+#define RANGES_DIAGNOSTIC_IGNORE(X) RANGES_DIAGNOSTIC_IGNORE_PRAGMAS RANGES_PRAGMA(GCC diagnostic ignored "-Wunknown-pragmas") RANGES_PRAGMA(GCC diagnostic ignored X)
 #define RANGES_DIAGNOSTIC_IGNORE_SHADOWING RANGES_DIAGNOSTIC_IGNORE("-Wshadow")
-#define RANGES_DIAGNOSTIC_IGNORE_PRAGMAS RANGES_DIAGNOSTIC_IGNORE("-Wunknown-pragmas") RANGES_DIAGNOSTIC_IGNORE("-Wpragmas")
 #define RANGES_DIAGNOSTIC_IGNORE_INDENTATION RANGES_DIAGNOSTIC_IGNORE("-Wmisleading-indentation")
 #define RANGES_DIAGNOSTIC_IGNORE_UNDEFINED_INTERNAL RANGES_DIAGNOSTIC_IGNORE("-Wundefined-internal")
 #define RANGES_DIAGNOSTIC_IGNORE_MISMATCHED_TAGS RANGES_DIAGNOSTIC_IGNORE("-Wmismatched-tags")
 #define RANGES_DIAGNOSTIC_IGNORE_SIGN_CONVERSION RANGES_DIAGNOSTIC_IGNORE("-Wsign-conversion")
 #define RANGES_DIAGNOSTIC_IGNORE_FLOAT_EQUAL RANGES_DIAGNOSTIC_IGNORE("-Wfloat-equal")
-#ifdef __clang__
+#define RANGES_DIAGNOSTIC_IGNORE_MISSING_BRACES RANGES_DIAGNOSTIC_IGNORE("-Wmissing-braces")
 #define RANGES_DIAGNOSTIC_IGNORE_GLOBAL_CONSTRUCTORS RANGES_DIAGNOSTIC_IGNORE("-Wglobal-constructors")
 #define RANGES_DIAGNOSTIC_IGNORE_UNNEEDED_INTERNAL RANGES_DIAGNOSTIC_IGNORE("-Wunneeded-internal-declaration")
 #define RANGES_DIAGNOSTIC_IGNORE_UNNEEDED_MEMBER RANGES_DIAGNOSTIC_IGNORE("-Wunneeded-member-function")
 #define RANGES_DIAGNOSTIC_IGNORE_ZERO_LENGTH_ARRAY RANGES_DIAGNOSTIC_IGNORE("-Wzero-length-array")
-#define RANGES_DIAGNOSTIC_IGNORE_CXX17_COMPAT
-#else
-#define RANGES_DIAGNOSTIC_IGNORE_GLOBAL_CONSTRUCTORS
-#define RANGES_DIAGNOSTIC_IGNORE_UNNEEDED_INTERNAL
-#define RANGES_DIAGNOSTIC_IGNORE_UNNEEDED_MEMBER
-#define RANGES_DIAGNOSTIC_IGNORE_ZERO_LENGTH_ARRAY
 #define RANGES_DIAGNOSTIC_IGNORE_CXX17_COMPAT RANGES_DIAGNOSTIC_IGNORE("-Wc++1z-compat")
-#endif
+#define RANGES_DIAGNOSTIC_IGNORE_UNDEFINED_FUNC_TEMPLATE RANGES_DIAGNOSTIC_IGNORE("-Wundefined-func-template")
+#define RANGES_DIAGNOSTIC_IGNORE_INCONSISTENT_OVERRIDE RANGES_DIAGNOSTIC_IGNORE("-Winconsistent-missing-override")
+#define RANGES_DIAGNOSTIC_IGNORE_RANGE_LOOP_ANALYSIS RANGES_DIAGNOSTIC_IGNORE("-Wrange-loop-analysis")
 #else
 #define RANGES_DIAGNOSTIC_PUSH
 #define RANGES_DIAGNOSTIC_POP
-#define RANGES_DIAGNOSTIC_IGNORE_SHADOWING
 #define RANGES_DIAGNOSTIC_IGNORE_PRAGMAS
+#define RANGES_DIAGNOSTIC_IGNORE_SHADOWING
 #define RANGES_DIAGNOSTIC_IGNORE_INDENTATION
 #define RANGES_DIAGNOSTIC_IGNORE_UNDEFINED_INTERNAL
 #define RANGES_DIAGNOSTIC_IGNORE_MISMATCHED_TAGS
@@ -239,6 +246,9 @@ namespace ranges
 #define RANGES_DIAGNOSTIC_IGNORE_ZERO_LENGTH_ARRAY
 #define RANGES_DIAGNOSTIC_IGNORE_CXX17_COMPAT
 #define RANGES_DIAGNOSTIC_IGNORE_FLOAT_EQUAL
+#define RANGES_DIAGNOSTIC_IGNORE_MISSING_BRACES
+#define RANGES_DIAGNOSTIC_IGNORE_UNDEFINED_FUNC_TEMPLATE
+#define RANGES_DIAGNOSTIC_IGNORE_INCONSISTENT_OVERRIDE
 #endif
 #endif // MSVC/Generic configuration switch
 
@@ -412,10 +422,17 @@ namespace ranges
     }
 #endif // RANGES_CXX_INLINE_VARIABLES
 
+#ifndef RANGES_CXX_DEDUCTION_GUIDES
+#ifdef __cpp_deduction_guides
+#define RANGES_CXX_DEDUCTION_GUIDES __cpp_deduction_guides
+#else
+#define RANGES_CXX_DEDUCTION_GUIDES RANGES_CXX_FEATURE(DEDUCTION_GUIDES)
+#endif // __cpp_deduction_guides
+#endif // RANGES_CXX_DEDUCTION_GUIDES
+
 #ifdef RANGES_FEWER_WARNINGS
 #define RANGES_DISABLE_WARNINGS                 \
     RANGES_DIAGNOSTIC_PUSH                      \
-    RANGES_DIAGNOSTIC_IGNORE_PRAGMAS            \
     RANGES_DIAGNOSTIC_IGNORE_SHADOWING          \
     RANGES_DIAGNOSTIC_IGNORE_UNDEFINED_INTERNAL \
     RANGES_DIAGNOSTIC_IGNORE_INDENTATION        \
@@ -427,6 +444,33 @@ namespace ranges
 #define RANGES_RE_ENABLE_WARNINGS
 #endif
 
+#if defined(__clang__)
+#if __has_attribute(no_sanitize)
+#define RANGES_INTENDED_MODULAR_ARITHMETIC \
+  __attribute__((__no_sanitize__("unsigned-integer-overflow")))
+#else
+#define RANGES_INTENDED_MODULAR_ARITHMETIC
+#endif
+#else
+#define RANGES_INTENDED_MODULAR_ARITHMETIC
+#endif
+
+#ifndef RANGES_CXX_IF_CONSTEXPR
+#ifdef __cpp_if_constexpr
+#define RANGES_CXX_IF_CONSTEXPR __cpp_if_constexpr
+#else
+#define RANGES_CXX_IF_CONSTEXPR RANGES_CXX_FEATURE(IF_CONSTEXPR)
+#endif
+#endif // RANGES_CXX_IF_CONSTEXPR
+
+#ifndef RANGES_CONSTEXPR_IF
+#if RANGES_CXX_IF_CONSTEXPR >= RANGES_CXX_IF_CONSTEXPR_17
+#define RANGES_CONSTEXPR_IF constexpr
+#else
+#define RANGES_CONSTEXPR_IF
+#endif
+#endif // RANGES_CONSTEXPR_IF
+
 namespace ranges {
     inline namespace v3 {
         namespace detail {
