@@ -1,7 +1,7 @@
 /// \file
 // Range v3 library
 //
-//  Copyright Eric Niebler 2013-2014
+//  Copyright Eric Niebler 2013-present
 //
 //  Use, modification and distribution is subject to the
 //  Boost Software License, Version 1.0. (See accompanying
@@ -44,6 +44,10 @@ namespace ranges
         struct random_access_iterator_tag
           : bidirectional_iterator_tag
         {};
+
+        struct contiguous_iterator_tag
+          : random_access_iterator_tag
+        {};
         /// @}
 
         /// \cond
@@ -107,7 +111,7 @@ namespace ranges
 
             ////////////////////////////////////////////////////////////////////////////////////////
             template<typename T>
-            meta::if_<std::is_object<T>, ranges::random_access_iterator_tag>
+            meta::if_<std::is_object<T>, ranges::contiguous_iterator_tag>
             iterator_category_helper(T **);
 
             template<typename T>
@@ -178,10 +182,12 @@ namespace ranges
                 using reference_t = Readable::reference_t<Out>;
 
                 template<typename Out, typename T>
-                auto requires_(Out& o, T &&t) -> decltype(
+                auto requires_(Out&& o, T &&t) -> decltype(
                     concepts::valid_expr(
                         ((void)(*o = (T &&) t), 42),
-                        ((void)(const_cast<reference_t<Out> const &&>(*o) = (T &&)t), 42)
+                        ((void)(*((Out &&) o) = (T &&) t), 42),
+                        ((void)(const_cast<reference_t<Out> const &&>(*o) = (T &&)t), 42),
+                        ((void)(const_cast<reference_t<Out> const &&>(*((Out &&) o)) = (T &&)t), 42)
                     ));
             };
 
@@ -368,6 +374,18 @@ namespace ranges
                         concepts::model_of<Same, reference_t<I>, decltype(i[i - i])>()
                     ));
             };
+
+            struct ContiguousIterator
+              : refines<RandomAccessIterator>
+            {
+                template<typename I>
+                auto requires_(I i) -> decltype(
+                    concepts::valid_expr(
+                        concepts::model_of<DerivedFrom, category_t<I>, ranges::contiguous_iterator_tag>(),
+                        concepts::is_true(std::is_lvalue_reference<reference_t<I>>{}),
+                        concepts::model_of<Same, value_t<I>, uncvref_t<reference_t<I>>>()
+                    ));
+            };
         }
 
         template<typename T>
@@ -429,12 +447,16 @@ namespace ranges
         template<typename I>
         using RandomAccessIterator = concepts::models<concepts::RandomAccessIterator, I>;
 
+        template<typename I>
+        using ContiguousIterator = concepts::models<concepts::ContiguousIterator, I>;
+
         ////////////////////////////////////////////////////////////////////////////////////////////
         // iterator_concept
         template<typename T>
         using iterator_concept =
             concepts::most_refined<
                 meta::list<
+                    concepts::ContiguousIterator,
                     concepts::RandomAccessIterator,
                     concepts::BidirectionalIterator,
                     concepts::ForwardIterator,
@@ -507,7 +529,7 @@ namespace ranges
                         meta::bind_front<meta::quote<InvocableConcept>, C&>,
                         meta::quote<meta::strict_and>>,
                     Is...>>;
-            
+
             template<typename C, typename ...Is>
             using common_result_indirect_invocable_ = meta::and_<
                 indirect_invocable_<Invocable, C, Is...>,
@@ -545,32 +567,32 @@ namespace ranges
             CopyConstructible<C>>;
 
         ////////////////////////////////////////////////////////////////////////////////////////////
-        // indirect_result_of
+        // indirect_invoke_result
         /// \cond
-        namespace detail
-        {
-            template<typename Sig, typename = void>
-            struct indirect_result_of_
-            {
-            };
+        template<typename Fun, typename... Is>
+        using indirect_invoke_result_t =
+            meta::if_c<
+                meta::and_c<(bool) Readable<Is>()...>::value,
+                invoke_result_t<Fun, concepts::Readable::reference_t<Is>...>>;
 
-            template<typename Fun, typename... Is>
-            struct indirect_result_of_<
-                Fun(Is...),
-                meta::if_c<meta::and_c<(bool) Readable<Is>()...>::value>>
-              : meta::if_c<
-                    (bool) Invocable<Fun, concepts::Readable::reference_t<Is>...>(),
-                    meta::defer<concepts::Invocable::result_t, Fun, concepts::Readable::reference_t<Is>...>,
-                    meta::nil_>
-            {
-            };
-        }
+        template<typename Fun, typename... Is>
+        struct indirect_invoke_result
+          : meta::defer<indirect_invoke_result_t, Fun, Is...>
+        {};
+
+        ////////////////////////////////////////////////////////////////////////////////////////////
+        // indirect_result_of
+        template<typename Sig>
+        struct indirect_result_of
+        {};
+
+        template<typename Fun, typename... Is>
+        struct indirect_result_of<Fun(Is...)>
+          : meta::defer<indirect_invoke_result_t, Fun, Is...>
+        {};
 
         template<typename Sig>
-        using indirect_result_of = detail::indirect_result_of_<Sig>;
-
-        template<typename Sig>
-        using indirect_result_of_t = meta::_t<detail::indirect_result_of_<Sig>>;
+        using indirect_result_of_t = meta::_t<indirect_result_of<Sig>>;
 
         ////////////////////////////////////////////////////////////////////////////////////////////
         // Project struct, for "projecting" a Readable with a unary callable
@@ -580,7 +602,7 @@ namespace ranges
             template<typename I, typename Proj>
             struct projected_
             {
-                using reference = indirect_result_of_t<Proj &(I)>;
+                using reference = indirect_invoke_result_t<Proj &, I>;
                 using value_type = uncvref_t<reference>;
                 reference operator*() const;
             };
