@@ -1,7 +1,7 @@
 /// \file
 // Range v3 library
 //
-//  Copyright Eric Niebler 2013-2014
+//  Copyright Eric Niebler 2013-present
 //
 //  Use, modification and distribution is subject to the
 //  Boost Software License, Version 1.0. (See accompanying
@@ -151,23 +151,28 @@ namespace ranges
                 struct predicate_pred
                 {
                     semiregular_t<Pred> pred_;
+
+                    template<class S, CONCEPT_REQUIRES_(Sentinel<S, iterator_t<Rng>>())>
                     std::pair<bool, iterator_t<Rng>>
-                    operator()(iterator_t<Rng> cur, sentinel_t<Rng> end) const
+                    operator()(iterator_t<Rng> cur, S end) const
                     {
                         auto where = ranges::find_if_not(cur, end, std::ref(pred_));
-                        return std::make_pair(cur != where, where);
+                        return std::pair<bool, iterator_t<Rng>>{cur != where, where};
                     }
                 };
                 template<typename Rng>
                 struct element_pred
                 {
                     range_value_type_t<Rng> val_;
+
+                    template<class S, CONCEPT_REQUIRES_(Sentinel<S, iterator_t<Rng>>())>
                     std::pair<bool, iterator_t<Rng>>
-                    operator()(iterator_t<Rng> cur, sentinel_t<Rng> end) const
+                    operator()(iterator_t<Rng> cur, S end) const
                     {
-                        using P = std::pair<bool, iterator_t<Rng>>;
                         RANGES_EXPECT(cur != end);
-                        return *cur == val_ ? P{true, ranges::next(cur)} : P{false, cur};
+                        bool const match = *cur == val_;
+                        if (match) ++cur;
+                        return std::pair<bool, iterator_t<Rng>>{match, cur};
                     }
                 };
                 template<typename Rng, typename Sub>
@@ -175,25 +180,27 @@ namespace ranges
                 {
                     all_t<Sub> sub_;
                     range_difference_type_t<Sub> len_;
+
                     subrange_pred() = default;
                     subrange_pred(Sub && sub)
                       : sub_(all(static_cast<Sub&&>(sub))), len_(distance(sub_))
                     {}
+                    template<class S, CONCEPT_REQUIRES_(Sentinel<S, iterator_t<Rng>>())>
                     std::pair<bool, iterator_t<Rng>>
-                    operator()(iterator_t<Rng> cur, sentinel_t<Rng> end) const
+                    operator()(iterator_t<Rng> cur, S end) const
                     {
+                        using P = std::pair<bool, iterator_t<Rng>>;
                         RANGES_EXPECT(cur != end);
-                        if(SizedSentinel<sentinel_t<Rng>, iterator_t<Rng>>() &&
-                            distance(cur, end) < len_)
-                            return {false, cur};
+                        if(SizedSentinel<S, iterator_t<Rng>>() && distance(cur, end) < len_)
+                            return P{false, cur};
                         auto pat_cur = ranges::begin(sub_);
                         auto pat_end = ranges::end(sub_);
                         for(;; ++cur, ++pat_cur)
                         {
                             if(pat_cur == pat_end)
-                                return {true, cur};
+                                return P{true, cur};
                             if(cur == end || !(*cur == *pat_cur))
-                                return {false, cur};
+                                return P{false, cur};
                         }
                     }
                 };
@@ -202,9 +209,10 @@ namespace ranges
                 using FunctionConcept = meta::and_<
                     ForwardRange<Rng>,
                     Invocable<Fun&, iterator_t<Rng>, sentinel_t<Rng>>,
+                    Invocable<Fun&, iterator_t<Rng>, iterator_t<Rng>>,
                     CopyConstructible<Fun>,
                     ConvertibleTo<
-                        result_of_t<Fun&(iterator_t<Rng>, sentinel_t<Rng>)>,
+                        invoke_result_t<Fun&, iterator_t<Rng>, sentinel_t<Rng>>,
                         std::pair<bool, iterator_t<Rng>>>>;
 
                 template<typename Rng, typename Fun>
